#!/usr/bin/perl
#
# check_sensor_temp.pl - Nagios custom plugin to SNMP check temperature values of
#						 AKCP sensorProbe8 v 2.0 SP8363 devices.
#
# Prerequisites:	Connected above device,
#					Net-SNMP suite (e.g. net-snmp* RPMs:
#									$ rpm -qa|grep snmp
#									net-snmp-utils-5.3.1-14.el5
#									net-snmp-libs-5.3.1-14.el5
#									net-snmp-5.3.1-14.el5
#									net-snmp-perl-5.3.1-14.el5
#									)
#					Important, don't confuse Net::SNMP with SNMP CPAN module!
#					The latter, which links against the net-snmp libs is needed
#					and thus provides MIB parsing capabilities without further
#					module installation ado.
#					Whereas Net::SNMP is a pure Perl implementation which
#					requires use of numeric OIDs without help of a separate
#					MIB parser or some home brewn mapping.
#					Most important, get vendor MIB for above device
#					(included on docs and supplements CD-ROM of device)
#					Copy MIB to /usr/share/snmp/mibs/,
#					or wherever you configured your snmpd build to find it.
#
# Author:			Ralph Grothe, 20070920
#
# Warnings:			Use at own risk.

use strict;
#use warnings;

use SNMP;
use Getopt::Std;

my %NAGIOS_SERVICESTATE = (
							0 => 'OK',
							1 => 'WARNING',
							2 => 'CRITICAL',
							3 => 'UNKNOWN',
						  );

my %NAGIOS_SERVICESTATEID = reverse %NAGIOS_SERVICESTATE;

my %NAGIOS_ENV = map { $_=>$ENV{$_} } grep /^NAGIOS_/m, keys %ENV;

# caveat emptor, this SIG handler provokes UNKNOWN states even for OKs
# when run by nagios binary (maybe due to some EmbPerl idiosyncrasy?)
#
#$SIG{__DIE__} = sub { chomp(my $out = shift);
#					  printf "%s - %s\n", $NAGIOS_SERVICESTATE{3}, $out;
#					  exit 3
#					};
# use this kludge instead
sub nagios_die {
	chomp(my $out = shift);
	printf "%s - %s\n", $NAGIOS_SERVICESTATE{3}, $out;
	exit 3
}

my (%opts, @temps, @warns, @crits, @unknowns);

getopts('h?dvH:C:P:p:w:c:t:', \%opts) or usage();
usage() if $opts{h} or $opts{'?'};
close STDERR;

my $agent = $opts{H} ? $opts{H} 
				: exists $NAGIOS_ENV{NAGIOS_HOSTADDRESS} ?
					$NAGIOS_ENV{NAGIOS_HOSTADDRESS} : undef;

nagios_die "Require address of SNMP agent to query!\n" unless $agent;

$SNMP::debugging = $opts{v} = $opts{d} if $opts{d};
my $community = $opts{C} || 'public';
my $timeout = $opts{t} || 10;
$timeout *= 10**6;
my $version = $opts{P} || 1;
my $port = $opts{p} || 161;

# threshold range mangling
my @warn_thresh = $opts{w}=~/:/ ? (split /:/, $opts{w})[0,1] : $opts{w}
					if ($opts{w} && $opts{w} =~ /^[\d.:]+$/);
shift @warn_thresh unless $warn_thresh[0];
my @crit_thresh = $opts{c}=~/:/ ? (split /:/, $opts{c})[0,1] : $opts{c}
					if ($opts{c} && $opts{c} =~ /^[\d.:]+$/);
shift @crit_thresh unless $crit_thresh[0];

# Not yet clear whether this skips loading of all mibs 
# in /usr/share/snmp/mibs as is intended here?
#
SNMP::loadModules(qw(SPAGENT-MIB))
	or nagios_die "Loading of SPAGENT-MIB failed: $!\n";

my $snmp = SNMP::Session->new(DestHost  => $agent,
							  Community => $community,
							  Version   => $version,
							  Port		=> $port,
							  Timeout	=> $timeout,
							  # will auto-convert e.g. state enums
							  UseEnums	=> 1,
							 );
nagios_die "Couldn't create SNMP object!\n" unless $snmp;

my $varbind_temp = SNMP::Varbind->new(
			['SPAGENT-MIB::sensorProbeTempDegreeRaw',0,undef,'INTEGER']);

my $varbind_online = SNMP::Varbind->new(
			['SPAGENT-MIB::sensorProbeTempOnline',0,undef,'INTEGER']);

my $varlist = SNMP::VarList->new($varbind_online, $varbind_temp);

# probably useless use of die?
nagios_die "Couldn't stack PDU varlist!\n" 
	unless ref $varlist eq 'SNMP::VarList';

# some device may have several sensors,
# why we getnext() loop over the whole table till slot which isn't online
#
my ($i, $online, $temp) = (0, $snmp->get($varlist));
while ($online eq 'online') {
	$temp /= 10;
	my $state = 0;
	# awkward in-ranges tests which should be improved
	if (@crit_thresh) {
		if (@crit_thresh == 2) {
			$state = 2 if $temp < $crit_thresh[0];
			$state = 2 if ($crit_thresh[1] and $temp > $crit_thresh[1]);
		} 
		else {
			$state = 2 if $temp > $crit_thresh[0];
		}
	}
	if (@warn_thresh && !$state) {
		if (@warn_thresh == 2) {
			$state = 1 if $temp < $warn_thresh[0];
			$state = 1 if ($warn_thresh[1] and $temp > $warn_thresh[1]);;
		}
		else {
			$state = 1 if $temp > $warn_thresh[0];
		}
	}
	push @temps, [$temp, $state, $snmp->{ErrorStr}];
	printf "Temp Sensor No.%2u: %5.2f Deg C\n", $i++, $temp
		if $opts{v};
	($online, $temp) = $snmp->getnext($varlist);
}

@crits = map $_->[0], grep $_->[1] == 2, @temps;
@warns = map $_->[0], grep $_->[1] == 1, @temps;
@unknowns = map $_->[2], grep $_->[2], @temps;

if (@crits) {
	my $i=0;
	printf "SENSOR %s - temp exceeded: ", $NAGIOS_SERVICESTATE{2};
	printf 'T%u=%5.2f degC 'x@crits, map @$_, map [$i++,$_], @crits;
	print "\n";
	exit 2;
}
elsif (@warns) {
	my $i=0;
	printf "SENSOR %s - temp exceeded: ", $NAGIOS_SERVICESTATE{1};
	printf 'T%u=%5.2f degC 'x@warns, map @$_, map [$i++,$_], @warns;
	print "\n";
	exit 1;
}
elsif (@unknowns) {
	local $" = '; ';
	printf "SENSOR %s - plugin failed: ", $NAGIOS_SERVICESTATE{3};
	print "@unknowns\n";
	exit 3;
}
else {
	my $i=0;
	#printf "SENSOR %s - temp: ", $NAGIOS_SERVICESTATE{0};
	my $out = sprintf "SENSOR %s - temp: ", $NAGIOS_SERVICESTATE{0};
	$out .= sprintf 'T%u=%5.2f degC 'x@temps, map @$_, map [$i++,$_->[0]], @temps;
	print "$out\n";
	exit 0;
}

sub usage {

my ($dirname, $me) = $0 =~ m|(.*)/(.+)|;

print STDERR <<USAGE;
@{[qx(sed -n 2,5p < "$dirname/$me")]}

usage: $me [-h|-?] | [-v|-d][-H agent_addr][-C community]
					[-t timeout][-p port][-P snmp_vers]
					[-w warn_thresh][-c crit_thresh]

Options mimic some of those of the official check_snmp plugin
(so compare check_snmp -h, especially with regard to warn/crit ranges).

-h|-?           this screen
-v              add verbosity (will break output restrictions for a Nagios plugin)
-d              includes -v plus debug level 1 messages from Perl SNMP module
-H agent_addr   address of SNMP agent to query (preferably IP)
-C community    community string for get requests of queried device (defaults to public)
-t timeout      timeout in secs (this plugin only uses synchronous requests)
-P snmp_vers    SNMP protocol version (boils really down to SNMPv1, dictated by HW)
-p port         UDP port of queried agent (defaults to 161 if ommitted)
-w low:high     thresholds that limit range for which no warning is thrown
-c low:high     same, but for critical check results

USAGE
exit 1;
}
